@@ -1,7 +1,7 @@
 '''Cobalt proxy provides client access to cobalt components'''
-__revision__ = '$Revision: 2954 $'
+__revision__ = '$Revision: 4291 $'
 
-import logging, socket, time, xmlrpclib, ConfigParser, httplib
+import logging, random, socket, time, xmlrpclib, ConfigParser, httplib
 from Bcfg2.tlslite.integration.XMLRPCTransport import XMLRPCTransport
 from Bcfg2.tlslite.integration.HTTPTLSConnection import HTTPTLSConnection
 from Bcfg2.tlslite.TLSConnection import TLSConnection
@@ -37,7 +37,7 @@ class MyXMLRPCTransport(XMLRPCTransport):
                                    self.checker.x509TrustList,
                                    self.checker.x509CommonName,
                                    self.settings)
-        http2 = httplib.HTTP()
+        http2 = httplib.HTTP(host)
         http2._setup(http)
         return http2
 
@@ -47,7 +47,7 @@ class CobaltComponentError(Exception):
 class SafeProxy:
     '''Wrapper for proxy'''
 
-    _retries = 4
+    _retries = 8
     _authinfo = ()
     _components = {}
     def __init__(self, component, args={}):
@@ -139,9 +139,6 @@ class SafeProxy:
             except xmlrpclib.Fault:
                 self.log.debug("Operation %s completed with fault" % (methodName))
                 raise
-            except socket.sslerror:
-                self.log.debug("Attempt %d of %d failed due to SSL negotiation failure" %
-                               ((irs + 1), self._retries))
             except socket.error, serr:
                 self.log.debug("Attempting %s (%d of %d) failed because %s" % (methodName, (irs+1),
                                                                                self._retries, serr))
@@ -150,10 +147,14 @@ class SafeProxy:
                 errmsg = err.message.split()
                 self.log.error("Got %s expected %s" % (errmsg[3], errmsg[4]))
                 raise SystemExit, 1
+            except Bcfg2.tlslite.errors.TLSLocalAlert, err:
+                self.log.error("TLS error; retrying: %s" % (err.message))
+                self.log.error("Check RX/TX hardware checksums")
             except:
                 self.log.error("Unknown failure", exc_info=1)
                 break
-            time.sleep(0.5)
+            stime = (irs * irs * 2 * random.random()) + 0.5
+            time.sleep(stime)
         self.log.error("%s failed:\nCould not connect to %s" % (methodName, self.component))
         raise xmlrpclib.Fault(20, "Server Failure")
         
@@ -182,7 +183,7 @@ class ComponentProxy(SafeProxy):
     name = 'dummy'
     methods = []
 
-    def __init__(self, url=None):
+    def __init__(self, url={}):
         SafeProxy.__init__(self, self.name, url)
         for method in self.methods:
             setattr(self, method, eval('lambda *x:self.run_method(method, x)',
