@@ -1,17 +1,27 @@
 # This is the bcfg2 support for blastwave packages (pkg-get)
 '''This provides bcfg2 support for blastwave'''
-__revision__ = '$Revision: 2401 $'
+__revision__ = '$Revision: 2768 $'
 
-import Bcfg2.Client.Tools.SYSV
+import Bcfg2.Client.Tools.SYSV, tempfile
 
 class Blast(Bcfg2.Client.Tools.SYSV.SYSV):
     '''Support for Blastwave packages'''
     pkgtype = 'blast'
-    pkgtool = ("/opt/csw/bin/pkg-get install %s", ("%s", ["name"]))
+    pkgtool = ("/opt/csw/bin/pkg-get install %s", ("%s", ["bname"]))
     __name__ = 'Blast'
     __execs__ = ['/opt/csw/bin/pkg-get']
     __handles__ = [('Package', 'blast')]
+    __ireq__ = {'Package': ['name', 'version', 'bname']}
 
+    def __init__(self, logger, setup, config, states):
+        # dont use the sysv constructor
+        Bcfg2.Client.Tools.PkgTool.__init__(self, logger, setup, config, states)
+        self.noaskname = tempfile.mktemp()
+        try:
+            open(self.noaskname, 'w+').write(Bcfg2.Client.Tools.SYSV.noask)
+        except:
+            pass
+        
     # VerifyPackage comes from Bcfg2.Client.Tools.SYSV
     # Install comes from Bcfg2.Client.Tools.PkgTool
     # Extra comes from Bcfg2.Client.Tools.Tool
