@@ -1,8 +1,8 @@
 # This is the bcfg2 support for chkconfig
-# $Id: Chkconfig.py 3021 2007-04-06 19:05:36Z desai $
+# $Id: Chkconfig.py 4237 2008-01-11 03:31:31Z desai $
 
 '''This is chkconfig support'''
-__revision__ = '$Revision: 3021 $'
+__revision__ = '$Revision: 4237 $'
 
 import Bcfg2.Client.Tools, Bcfg2.Client.XML
 
@@ -16,27 +16,35 @@ class Chkconfig(Bcfg2.Client.Tools.SvcTool):
     def VerifyService(self, entry, _):
         '''Verify Service status for entry'''
         try:
-            srvdata = self.cmd.run('/sbin/chkconfig --list %s | grep -v "unknown service"'
-                                   % entry.attrib['name'])[1][0].split()
+            cmd = "/sbin/chkconfig --list %s " % (entry.get('name'))
+            raw = self.cmd.run(cmd)[1]
+            patterns = ["error reading information", "unknown service"]
+            srvdata = [line.split() for line in raw for pattern in patterns \
+                       if pattern not in line][0]
         except IndexError:
             # Ocurrs when no lines are returned (service not installed)
             entry.set('current_status', 'off')
             return False
-        if entry.attrib['type'] == 'xinetd':
-            return entry.attrib['status'] == srvdata[1]
+        if entry.get('type') == 'xinetd':
+            return entry.get('status') == srvdata[1]
 
         try:
-            onlevels = [level.split(':')[0] for level in srvdata[1:] if level.split(':')[1] == 'on']
+            onlevels = [level.split(':')[0] for level in srvdata[1:] \
+                        if level.split(':')[1] == 'on']
         except IndexError:
             onlevels = []
 
-        pstatus = self.cmd.run('/sbin/service %s status' % \
-                               entry.get('name'))[0]
-        # chkconfig/init.d service
         if entry.get('status') == 'on':
-            status = (len(onlevels) > 0 ) and not pstatus
+            status = (len(onlevels) > 0 )
         else:
-            status = (len(onlevels) == 0) and pstatus
+            status = (len(onlevels) == 0)
+
+        if entry.get('supervised', 'false') == 'true':
+            pstatus, pout = self.cmd.run('/sbin/service %s status' % \
+                                         entry.get('name'))
+            # chkconfig/init.d service
+            if entry.get('status') == 'on':
+                status = status and not pstatus
 
         if not status:
             if entry.get('status') == 'on':
@@ -47,10 +55,15 @@ class Chkconfig(Bcfg2.Client.Tools.SvcTool):
 
     def InstallService(self, entry):
         '''Install Service entry'''
+        rcmd = "/sbin/chkconfig %s %s" 
         self.cmd.run("/sbin/chkconfig --add %s"%(entry.attrib['name']))
         self.logger.info("Installing Service %s" % (entry.get('name')))
-        return self.cmd.run("/sbin/chkconfig %s %s" % (entry.get('name'),
-                                                       entry.get('status')))[0] == 0
+        pass1 = True
+        if entry.get('status') == 'off':
+            rc = self.cmd.run(rcmd % (entry.get('name'), entry.get('status')) + " --level 1")[0]
+            pass1 = rc == 0
+        rc = self.cmd.run(rcmd % (entry.get('name'), entry.get('status')))[0]
+        return pass1 and rc == 0
 
     def FindExtra(self):
         '''Locate extra chkconfig Services'''
