@@ -1,5 +1,5 @@
 '''SMF support for Bcfg2'''
-__revision__ = '$Revision: 2401 $'
+__revision__ = '$Revision: 2632 $'
 
 import glob, os
 import Bcfg2.Client.Tools
@@ -23,10 +23,12 @@ class SMF(Bcfg2.Client.Tools.Tool):
             else:
                 self.logger.info('Failed to locate FMRI for service %s' % entry.get('name'))
                 return False
+        return True
     
     def VerifyService(self, entry, _):
         '''Verify SMF Service Entry'''
         if not self.GetFMRI(entry):
+            self.logger.error("smf service %s doesn't have FMRI set" % entry.get('name'))
             return False
         if entry.get('FMRI').startswith('lrc'):
             filename = entry.get('FMRI').split('/')[-1]
@@ -41,7 +43,7 @@ class SMF(Bcfg2.Client.Tools.Tool):
                 self.logger.debug("No service matching %s" % (entry.get("FMRI")))
                 return entry.get('status') == 'off'
         try:
-            srvdata = self.cmd.run("/usr/bin/svcs -H -o STA %s" % entry.attrib['name'])[1][0].split()
+            srvdata = self.cmd.run("/usr/bin/svcs -H -o STA %s" % entry.get('FMRI'))[1][0].split()
         except IndexError:
             # Ocurrs when no lines are returned (service not installed)
             return False
@@ -70,7 +72,7 @@ class SMF(Bcfg2.Client.Tools.Tool):
                                      (entry.get('FMRI')))[0]
         else:
             if entry.get('FMRI').startswith('lrc'):
-                loc = entry.get("FMRI")[4:].replace('_', ',')
+                loc = entry.get("FMRI")[4:].replace('_', '.')
                 try:
                     os.stat(loc.replace('/S', '/Disabled.'))
                     self.logger.debug("Renaming file %s to %s" % \
@@ -82,32 +84,33 @@ class SMF(Bcfg2.Client.Tools.Tool):
                                       % (loc.replace('/S', '/DISABLED.S'), loc))
                     cmdrc = 1
             else:
-                cmdrc = self.cmd.run("/usr/sbin/svcadm enable -r %s" % \
-                                     (entry.get('FMRI')))[0]
+                srvdata = self.cmd.run("/usr/bin/svcs -H -o STA %s" %
+                                       entry.get('FMRI'))[1] [0].split()
+                if srvdata[0] == 'MNT':
+                    cmdarg = 'clear'
+                else:
+                    cmdarg = 'enable'
+                cmdrc = self.cmd.run("/usr/sbin/svcadm %s -r %s" % \
+                                     (cmdarg, entry.get('FMRI')))[0]
         return cmdrc == 0
 
     def Remove(self, svcs):
         '''Remove Extra SMF entries'''
+        # Extra service entry removal is nonsensical
+        # Extra service entries should be reflected in config, even if disabled
         pass
 
     def FindExtra(self):
         '''Find Extra SMF Services'''
         allsrv = [name for name, version in \
-                  [ srvc.strip().split() for srvc in
-                    self.cmd.run("/usr/bin/svcs -a -H -o FMRI,STATE")[1] ]
+                  [srvc.split() for srvc in
+                   self.cmd.run("/usr/bin/svcs -a -H -o FMRI,STATE")[1]]
                   if version != 'disabled']
 
-        for svc in self.getSupportedEntries():
-            name = self.cmd.run("/usr/bin/svcs -H -o FMRI %s 2>/dev/null" % \
-                                svc.get('name'))[1]
-            if name:
-                svc.set('FMRI', name[0])
-                if name in allsrv:
-                    allsrv.remove(name)
-            else:
-                self.logger.info("Failed to locate FMRI for service %s" % svc.get('name'))
-                
-        return [Bcfg2.Client.XML.Element("Service", type='smf', name=name) for name in allsrv]
+        [allsrv.remove(svc.get('FMRI')) for svc in self.getSupportedEntries() \
+         if svc.get("FMRI") in allsrv]
+        return [Bcfg2.Client.XML.Element("Service", type='smf', name=name) \
+                for name in allsrv]
 
     def BundleUpdated(self, bundle):
         '''Restart smf services'''
@@ -115,9 +118,15 @@ class SMF(Bcfg2.Client.Tools.Tool):
             if not self.canInstall(entry):
                 self.logger.error("Insufficient information to restart service %s" % (entry.get('name')))
             else:
-                if entry.get('status') == 'on':
-                    self.logger.info("Restarting smf service %s" % (entry.get("FMRI")))
-                    self.cmd.run("svcadm restart %s" % (entry.get("FMRI")))
+                if entry.get("FMRI").startswith('lrc'):
+                    if entry.get('status') == 'on':
+                        self.logger.info("Restarting smf/lrc service %s"%(entry.get("name")))
+                        self.cmd.run("/etc/init.d/%s %s" % (entry.get('name'),
+                                                            entry.get('reload', 'reload')))
                 else:
-                    self.cmd.run("svcadm disable %s" % (entry.get("FMRI")))
+                    if entry.get('status') == 'on':
+                        self.logger.info("Restarting smf service %s" % (entry.get("FMRI")))
+                        self.cmd.run("/usr/sbin/svcadm restart %s" % (entry.get("FMRI")))
+                    else:
+                        self.cmd.run("/usr/sbin/svcadm disable %s" % (entry.get("FMRI")))
 
