@@ -1,6 +1,6 @@
 # This is the bcfg2 support for solaris sysv packages
 '''This provides bcfg2 support for Solaris SYSV packages'''
-__revision__ = '$Revision: 2888 $'
+__revision__ = '$Revision: 4210 $'
 
 import tempfile, Bcfg2.Client.Tools, Bcfg2.Client.XML
 
@@ -24,9 +24,10 @@ class SYSV(Bcfg2.Client.Tools.PkgTool):
     __execs__ = ["/usr/sbin/pkgadd"]
     __handles__ = [('Package', 'sysv')]
     __req__ = {'Package': ['name', 'version']}
+    __ireq__ = {'Package': ['name', 'url', 'version']}
     __name__ = 'SYSV'
     pkgtype = 'sysv'
-    pkgtool = ("/usr/sbin/pkgadd %s -d %%s -n %%%%s", (("%s", ["name"])))
+    pkgtool = ("/usr/sbin/pkgadd %s -n -d %%s", (('%s %s', ['url', 'name'])))
 
     def __init__(self, logger, setup, config, states):
         Bcfg2.Client.Tools.PkgTool.__init__(self, logger, setup, config, states)
