@@ -1,8 +1,10 @@
 '''Bcfg2 logging support'''
-__revision__ = '$Revision: 2542 $'
+__revision__ = '$Revision: 2975 $'
 
 import copy, fcntl, logging, logging.handlers, math, socket, struct, sys, termios, types
 
+logging.raiseExceptions=0
+
 def print_attributes(attrib):
     ''' Add the attributes for an element'''
     return ' '.join(['%s="%s"' % data for data in attrib.iteritems()])
@@ -125,7 +127,7 @@ class FragmentingSysLogHandler(logging.handlers.SysLogHandler):
                 self.socket.send("Reconnected to syslog")
                 self.socket.send(msg)
 
-def setup_logging(procname, to_console=True, to_syslog=True, syslog_facility='local0', level=0):
+def setup_logging(procname, to_console=True, to_syslog=True, syslog_facility='daemon', level=0):
     '''setup logging for bcfg2 software'''
     if hasattr(logging, 'already_setup'):
         return 
