@@ -1,5 +1,5 @@
 '''Bcfg2.Server.Core provides the runtime support for bcfg2 modules'''
-__revision__ = '$Revision: 2614 $'
+__revision__ = '$Revision: 3903 $'
 
 from time import time
 from Bcfg2.Server.Plugin import PluginInitError, PluginExecutionError
@@ -103,7 +103,7 @@ class GaminEvent(object):
     '''This class provides an event analogous to python-fam events based on gamin sources'''
     def __init__(self, request_id, filename, code):
         action_map = {GAMCreated: 'created', GAMExists: 'exists', GAMChanged: 'changed',
-                      GAMDeleted: 'deleted', GAMEndExist: 'endExist'}
+                      GAMDeleted: 'deleted', GAMEndExist: 'endExist', GAMMoved: 'moved'}
         self.requestID = request_id
         self.filename = filename
         if action_map.has_key(code):
@@ -185,7 +185,7 @@ class GaminFam(object):
         return count
         
 try:
-    from gamin import WatchMonitor, GAMCreated, GAMExists, GAMEndExist, GAMChanged, GAMDeleted
+    from gamin import WatchMonitor, GAMCreated, GAMExists, GAMEndExist, GAMChanged, GAMDeleted, GAMMoved
     monitor = GaminFam
 except ImportError:
     # fall back to _fam
@@ -223,8 +223,8 @@ class Core(object):
         mpath = self.cfile.get('server','repository')
         self.stats = Statistics("%s/etc/statistics.xml" % (mpath))
 
-        structures = self.cfile.get('server', 'structures').split(',')
-        generators = self.cfile.get('server', 'generators').split(',')
+        structures = self.cfile.get('server', 'structures').replace(' ', '').split(',')
+        generators = self.cfile.get('server', 'generators').replace(' ', '').split(',')
         [data.remove('') for data in [structures, generators] if '' in data]
 
         for plugin in structures + generators + ['Metadata']:
@@ -232,8 +232,8 @@ class Core(object):
                 try:
                     mod = getattr(__import__("Bcfg2.Server.Plugins.%s" %
                                              (plugin)).Server.Plugins, plugin)
-                except ImportError:
-                    logger.error("Failed to load plugin %s" % (plugin))
+                except ImportError, e:
+                    logger.error("Failed to load plugin %s: %s" % (plugin, e))
                     continue
                 struct = getattr(mod, plugin)
                 try:
@@ -241,7 +241,7 @@ class Core(object):
                 except PluginInitError:
                     logger.error("Failed to instantiate plugin %s" % (plugin))
                 except:
-                    logger.error("Unexpected initiantiation failure for plugin %s" % (plugin), exc_info=1)
+                    logger.error("Unexpected instantiation failure for plugin %s" % (plugin), exc_info=1)
 
         self.metadata = self.plugins['Metadata']
         for plugin in structures:
@@ -273,6 +273,24 @@ class Core(object):
 
     def Bind(self, entry, metadata):
         '''Bind an entry using the appropriate generator'''
+        if 'altsrc' in entry.attrib:
+            oldname = entry.get('name')
+            entry.set('name', entry.get('altsrc'))
+            entry.set('realname', oldname)
+            del entry.attrib['altsrc']
+            try:
+                ret = self.Bind(entry, metadata)
+                entry.set('name', oldname)
+                del entry.attrib['realname']
+                return ret
+            except:
+                entry.set('name', oldname)
+                logger.error("Failed binding entry %s:%s with altsrc %s" \
+                             % (entry.tag, entry.get('name'),
+                                entry.get('altsrc')))
+                logger.error("Falling back to %s:%s" % (entry.tag,
+                                                        entry.get('name')))
+
         glist = [gen for gen in self.generators if
                  gen.Entries.get(entry.tag, {}).has_key(entry.get('name'))]
         if len(glist) == 1:
@@ -307,6 +325,18 @@ class Core(object):
             # do prereq processing
             prereqs = self.plugins['Deps'].GeneratePrereqs(structures, meta)
             structures.append(prereqs)
+
+        # Perform altsrc consistency checking
+        esrcs = {}
+        for struct in structures:
+            for entry in struct:
+                key = (entry.tag, entry.get('name'))
+                if key in esrcs:
+                    if esrcs[key] != entry.get('altsrc'):
+                        logger.error("Found inconsistent altsrc mapping for entry %s:%s" % key)
+                else:
+                    esrcs[key] = entry.get('altsrc', None)
+        del esrcs
         
         for astruct in structures:
             try:
