@@ -1,14 +1,14 @@
 '''This module sets up a base list of configuration entries'''
-__revision__ = '$Revision: 2341 $'
+__revision__ = '$Revision: 2695 $'
 
 import Bcfg2.Server.Plugin
 import copy
 import lxml.etree
 
-class Base(Bcfg2.Server.Plugin.Plugin, Bcfg2.Server.Plugin.DirectoryBacked):
+class Base(Bcfg2.Server.Plugin.Plugin, Bcfg2.Server.Plugin.XMLDirectoryBacked):
     '''This Structure is good for the pile of independent configs needed for most actual systems'''
     __name__ =  'Base'
-    __version__ = '$Id: Base.py 2341 2006-09-28 22:07:53Z desai $'
+    __version__ = '$Id: Base.py 2695 2007-01-18 16:41:20Z desai $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     __child__ = Bcfg2.Server.Plugin.StructFile
     
@@ -17,7 +17,7 @@ class Base(Bcfg2.Server.Plugin.Plugin, Bcfg2.Server.Plugin.DirectoryBacked):
         Bcfg2.Server.Plugin.Plugin.__init__(self, core, datastore)
         self.fragements = {}
         try:
-            Bcfg2.Server.Plugin.DirectoryBacked.__init__(self, self.data, self.core.fam)
+            Bcfg2.Server.Plugin.XMLDirectoryBacked.__init__(self, self.data, self.core.fam)
         except OSError:
             self.logger.error("Failed to load Base repository")
             raise Bcfg2.Server.Plugin.PluginInitError
