@@ -1,19 +1,19 @@
 '''This provides bundle clauses with translation functionality'''
-__revision__ = '$Revision: 1717 $'
+__revision__ = '$Revision: 2695 $'
 
 import copy, lxml.etree, Bcfg2.Server.Plugin
 
-class Bundler(Bcfg2.Server.Plugin.Plugin, Bcfg2.Server.Plugin.DirectoryBacked):
+class Bundler(Bcfg2.Server.Plugin.Plugin, Bcfg2.Server.Plugin.XMLDirectoryBacked):
     '''The bundler creates dependent clauses based on the bundle/translation scheme from bcfg1'''
     __name__ =  'Bundler'
-    __version__ = '$Id: Bundler.py 1717 2006-01-25 16:48:06Z desai $'
+    __version__ = '$Id: Bundler.py 2695 2007-01-18 16:41:20Z desai $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     __child__ = Bcfg2.Server.Plugin.StructFile
     
     def __init__(self, core, datastore):
         Bcfg2.Server.Plugin.Plugin.__init__(self, core, datastore)
         try:
-            Bcfg2.Server.Plugin.DirectoryBacked.__init__(self, self.data, self.core.fam)
+            Bcfg2.Server.Plugin.XMLDirectoryBacked.__init__(self, self.data, self.core.fam)
         except OSError:
             self.logger.error("Failed to load Bundle repository")
             raise Bcfg2.Server.Plugin.PluginInitError
