@@ -1,21 +1,15 @@
 '''This module implements a config file repository'''
-__revision__ = '$Revision: 3031 $'
+__revision__ = '$Revision: 4361 $'
 
 import binascii, difflib, logging, os, re, stat, tempfile, \
        xml.sax.saxutils, Bcfg2.Server.Plugin, lxml.etree
 
 logger = logging.getLogger('Bcfg2.Plugins.Cfg')
 
-specific = re.compile('(.*/)(?P<filename>[\S\-.]+)\.((H_(?P<hostname>\S+))|' +
+specific = re.compile('(.*/)?(?P<filename>[\S\-.]+)\.((H_(?P<hostname>\S+))|' +
                       '(G(?P<prio>\d+)_(?P<group>\S+)))$')
 probeData = {}
 
-def update_file(path, diff):
-    '''Update file at path using diff'''
-    newdata = '\n'.join(difflib.restore(xml.sax.saxutils.unescape(diff).split('\n'), 1))
-    print "writing file, %s" % path
-    open(path, 'w').write(newdata)
-
 class SpecificityError(Exception):
     '''Thrown in case of filename parse failure'''
     pass
@@ -33,7 +27,9 @@ class FileEntry(Bcfg2.Server.Plugin.FileBacked):
         self.group = False
         self.op = False
         self.prio = False
-        if name.split('.')[-1] in ['cat', 'diff']:
+        bfname = name.split('/')[-1]
+        bid = myid.split('/')[-1]
+        if bfname[len(bid)+1:].split('.')[-1] in ['cat', 'diff']:
             self.op = name.split('.')[-1]
             name = name[:-(len(self.op) + 1)]
         if self.name.split('/')[-1] == myid.split('/')[-1]:
@@ -126,26 +122,34 @@ class ConfigFileEntry(object):
         '''add new file additions for a single cf file'''
         basename = name.split('/')[-1]
         rbasename = self.repopath.split('/')[-1]
-        if not ((basename in [':info', 'info']) or
+        if not ((basename in [':info', 'info', ':info.xml', 'info.xml']) or
                 (basename[:len(rbasename)] == rbasename)):
             logger.error("Confused about file %s; ignoring" % (name))
             return
         if basename in [':info', 'info']:
             return self.read_info(basename)
+        elif basename in ['info.xml', ':info.xml']:
+            fpath = self.repopath + '/' + basename
+            self.infoxml = Bcfg2.Server.Plugin.XMLSrc(fpath, True)
+            return
 
         try:
-            self.fragments.append(FileEntry(self.path, name))
-            self.fragments.sort()
+            if name not in [ent.name for ent in self.fragments]:
+                self.fragments.append(FileEntry(self.path, name))
+                self.fragments.sort()
         except SpecificityError:
             return
 
     def HandleEvent(self, event):
         '''Handle FAM updates'''
         action = event.code2str()
-        #logger.debug("Got event %s for %s" % (action, event.filename))
+        #if self.path.endswith('/etc/passwd'):
+        #    logger.debug("Got event %s for %s" % (action, event.filename))
         if event.filename in [':info', 'info']:
             if action in ['changed', 'exists', 'created']:
                 return self.read_info(event.filename)
+        elif event.filename in [':info.xml', 'info.xml']:
+            return self.infoxml.HandleEvent(event)
         if event.filename != self.path.split('/')[-1]:
             if not specific.match('/' + event.filename):
                 logger.info('Suppressing event for bogus file %s' % event.filename)
@@ -223,8 +227,19 @@ class ConfigFileEntry(object):
                 filedata = output
             else:
                 logger.error("Unknown delta type %s" % (delta.op))
-            
-        [entry.attrib.__setitem__(key, value) for (key, value) in self.metadata.iteritems()]
+
+        if hasattr(self, 'infoxml'):
+            mdata = {}
+            if not self.infoxml.pnode:
+                logger.error("Cfg: infoxml error for file %s" % \
+                             (entry.get('name')))
+                raise Bcfg2.Server.Plugin.PluginExecutionError, ('infoxml', name)
+            self.infoxml.pnode.Match(metadata, mdata)
+            mdata = mdata['Info'][None]
+        else:
+            mdata = self.metadata
+        [entry.attrib.__setitem__(key, value) \
+         for (key, value) in mdata.iteritems()]
         if self.interpolate:
             if metadata.hostname in probeData:
                 for name, value in probeData[metadata.hostname].iteritems():
@@ -234,7 +249,7 @@ class ConfigFileEntry(object):
                 logger.warning("Cannot interpolate data for client: %s for config file: %s"% (metadata.hostname, basefile.name))
         if self.paranoid:
             entry.attrib['paranoid'] = 'true'
-        if entry.attrib['encoding'] == 'base64':
+        if entry.get('encoding', 'ascii') == 'base64':
             entry.text = binascii.b2a_base64(filedata)
             return
         if filedata == '':
@@ -248,7 +263,7 @@ class ConfigFileEntry(object):
 class Cfg(Bcfg2.Server.Plugin.Plugin):
     '''This generator in the configuration file repository for bcfg2'''
     __name__ = 'Cfg'
-    __version__ = '$Id: Cfg.py 3031 2007-04-11 18:37:22Z desai $'
+    __version__ = '$Id: Cfg.py 4361 2008-02-11 22:47:31Z desai $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     tempfile = re.compile("^.*~$|^.*\.swp")
 
@@ -356,14 +371,32 @@ class Cfg(Bcfg2.Server.Plugin.Plugin):
         elif action == 'deleted':
             if self.entries.has_key(configfile):
                 self.entries[configfile].HandleEvent(event)
+                try:
+                    os.stat(self.entries[configfile].repopath)
+                except:
+                    logger.info("Removing service for ConfigFile %s" % \
+                                self.entries[configfile].path)
+                    del self.entries[configfile]
         elif action in ['exists', 'endExist']:
             pass
         else:
             logger.error("Got unknown event %s %s:%s" % (action, event.requestID, event.filename))
         self.interpolate = len([entry for entry in self.entries.values() if entry.interpolate ]) > 0
 
-    def AcceptEntry(self, meta, _, entry_name, diff):
+    def AcceptEntry(self, meta, _, entry_name, diff, fulldata, metadata_updates={}):
         '''per-plugin bcfg2-admin pull support'''
+        if metadata_updates:
+            if hasattr(self.Entries['ConfigFile'][entry_name], 'infoxml'):
+                print "InfoXML support not yet implemented"
+            elif raw_input("Should metadata updates apply to all hosts? (n/Y) ") in ['Y', 'y']:
+                self.entries[entry_name].metadata.update(metadata_updates)
+                infofile = open(self.entries[entry_name].repopath + '/:info', 'w')
+                for x in self.entries[entry_name].metadata.iteritems():
+                    infofile.write("%s: %s\n" % x)
+                infofile.close()
+        if not diff and not fulldata:
+            raise SystemExit, 0
+                
         hsq = "Found host-specific file %s; Should it be updated (n/Y): "
         repo_vers = lxml.etree.Element('ConfigFile', name=entry_name)
         self.Entries['ConfigFile'][entry_name](repo_vers, meta)
@@ -372,24 +405,34 @@ class Cfg(Bcfg2.Server.Plugin.Plugin):
         basefile = [frag for frag in \
                     self.entries[entry_name].fragments \
                     if frag.applies(meta)][-1]
-        gsq = "Should this change apply to this host of all hosts effected by file %s? (N/y): " % (basefile.name)
+        gsq = "Should this change apply to all hosts effected by file %s? (N/y): " % (basefile.name)
         if ".H_%s" % (meta.hostname) in basefile.name:
-            answer = raw_input(hsq)
+            answer = raw_input(hsq % basefile.name)
         else:
             answer = raw_input(gsq)
-
-        if answer in 'Yy':
-            update_file(basefile.name, diff)
+        
+        if answer in ['Y', 'y']:
+            print "writing file, %s" % basefile.name
+            if fulldata:
+                newdata = fulldata
+            else:
+                newdata = '\n'.join(difflib.restore(diff.split('\n'), 1))
+            open(basefile.name, 'w').write(newdata)
             return
 
         if ".H_%s" % (meta.hostname) in basefile.name:
             raise SystemExit, 1
         # figure out host-specific filename
-        if '.G_' in basefile.name:
-            idx = basefile.name.find(".G_")
-            newname = basefile.name[:idx] + ".H_%s" % (meta.hostname)
+        reg = re.compile("(.*)\.G\d+.*")
+        if reg.match(basefile.name):
+            newname = reg.match(basefile.name).group(1) + ".H_%s" % (meta.hostname)
         else:
             newname = basefile.name + ".H_%s" % (meta.hostname)
         print "This file will be installed as file %s" % newname
-        if raw_input("Should it be installed? (N/y): ") in 'Yy':
-            update_file(newname, diff)
+        if raw_input("Should it be installed? (N/y): ") in ['Y', 'y']:
+            print "writing file, %s" % newname
+            if fulldata:
+                newdata = fulldata
+            else:
+                newdata = '\n'.join(difflib.restore(diff.split('\n'), 1))
+            open(newname, 'w').write(newdata)
