@@ -1,5 +1,5 @@
 '''This file stores persistent metadata for the BCFG Configuration Repository'''
-__revision__ = '$Revision: 3042 $'
+__revision__ = '$Revision: 3948 $'
 
 import lxml.etree, re, socket, time, sys, ConfigParser
 import Bcfg2.Server.Plugin
@@ -30,7 +30,7 @@ class ClientMetadata(object):
 
 class Metadata(Bcfg2.Server.Plugin.Plugin):
     '''This class contains data for bcfg2 server metadata'''
-    __version__ = '$Id: Metadata.py 3042 2007-04-16 18:02:41Z desai $'
+    __version__ = '$Id: Metadata.py 3948 2007-11-12 19:31:22Z desai $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     __name__ = "Metadata"
 
@@ -57,19 +57,17 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
         self.session_cache = {}
         self.clientdata = None
         self.default = None
+        self.pdirty = False
         try:
-            self.probes = Bcfg2.Server.Plugin.DirectoryBacked(datastore + "/Probes",
-                                                              core.fam)
+            loc = datastore + "/Probes"
+            self.probes = Bcfg2.Server.Plugin.DirectoryBacked(loc, core.fam)
         except:
             self.probes = False
         self.probedata = {}
+        self.ptimes = {}
+        self.pctime = 0
         self.extra = {'groups.xml':[], 'clients.xml':[]}
-        CP = ConfigParser.ConfigParser()
-        if '-C' in sys.argv:
-            CP.read([sys.argv[sys.argv.index('-C') + 1]])
-        else:
-            CP.read(['/etc/bcfg2.conf'])
-        self.password = CP.get('communication', 'password')
+        self.password = core.cfile.get('communication', 'password')
 
     def HandleEvent(self, event):
         '''Handle update events for data files'''
@@ -110,28 +108,31 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
             self.addresses = {}
             self.clientdata = xdata
             for client in xdata.findall('.//Client'):
+                clname = client.get('name').lower()
                 if 'address' in client.attrib:
                     caddr = client.get('address')
                     if self.addresses.has_key(caddr):
-                        self.addresses[caddr].append(client.get('name'))
+                        self.addresses[caddr].append(clname)
                     else:
-                        self.addresses[caddr] = [client.get('name')]
+                        self.addresses[caddr] = [clname]
                 if 'uuid' in client.attrib:
-                    self.uuid[client.get('uuid')] = client.get('name')
+                    self.uuid[client.get('uuid')] = clname
                 if 'secure' in client.attrib:
-                    self.secure.append(client.get('name'))
+                    self.secure.append(clname)
                 if client.get('location', 'fixed') == 'floating':
-                    self.floating.append(client.get('name'))
+                    self.floating.append(clname)
                 if 'password' in client.attrib:
-                    self.passwords[client.get('name')] = client.get('password')
-                for alias in [alias for alias in client.findall('Alias') if 'address' in alias.attrib]:
+                    self.passwords[clname] = client.get('password')
+                for alias in [alias for alias in client.findall('Alias')\
+                              if 'address' in alias.attrib]:
                     if self.addresses.has_key(alias.get('address')):
-                        self.addresses[alias.get('address')].append(client.get('name'))
+                        self.addresses[alias.get('address')].append(clname)
                     else:
-                        self.addresses[alias.get('address')] = (client.get('name'))
+                        self.addresses[alias.get('address')] = [clname]
                     
-                self.clients.update({client.get('name'): client.get('profile')})
-                [self.aliases.update({alias.get('name'): client.get('name')}) for alias in client.findall('Alias')]
+                self.clients.update({clname: client.get('profile')})
+                [self.aliases.update({alias.get('name'): clname}) \
+                 for alias in client.findall('Alias')]
         elif dest == 'groups.xml':
             self.public = []
             self.profiles = []
@@ -139,9 +140,8 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
             self.groups = {}
             grouptmp = {}
             self.categories = {}
-            for group in xdata.findall('.//Group'):
-                if group.getparent().tag == 'Group':
-                    continue
+            for group in xdata.xpath('//Groups/Group') \
+                    + xdata.xpath('Group'):
                 grouptmp[group.get('name')] = tuple([[item.get('name') for item in group.findall(spec)]
                                                      for spec in ['./Bundle', './Group']])
                 grouptmp[group.get('name')][1].append(group.get('name'))
@@ -226,6 +226,36 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
         datafile.write(lxml.etree.tostring(self.clientdata.getroot()))
         datafile.close()
 
+    def write_probedata(self):
+        '''write probe data out for use with bcfg2-info'''
+        if self.pdirty:
+            top = lxml.etree.Element("Probed")
+            for client, probed in self.probedata.iteritems():
+                cx = lxml.etree.SubElement(top, 'Client', name=client)
+                for probe in probed:
+                    lxml.etree.SubElement(cx, 'Probe', name=probe,
+                                          value=self.probedata[client][probe])
+            data = lxml.etree.tostring(top)
+            try:
+                datafile = open("%s/%s" % (self.data, 'probed.xml'), 'w')
+            except IOError:
+                self.logger.error("Failed to write clients.xml")
+                raise MetadataRuntimeError
+            datafile.write(data)
+            self.pdirty = False
+
+    def load_probedata(self):
+        try:
+            data = lxml.etree.parse(self.data + '/probed.xml').getroot()
+        except:
+            self.logger.error("Failed to read file probed.xml")
+            return
+        self.probedata={}
+        for client in data.getchildren():
+            self.probedata[client.get('name')] = {}
+            for pdata in client:
+                self.probedata[client.get('name')][pdata.get('name')] = pdata.get('value')
+
     def find_toolset(self, client):
         '''Find the toolset for a given client'''
         tgroups = [self.toolsets[group] for group in self.groups[client][1] if self.toolsets.has_key(group)]
@@ -244,7 +274,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
 
     def resolve_client(self, addresspair):
         '''Lookup address locally or in DNS to get a hostname'''
-        print self.session_cache
+        #print self.session_cache
         if self.session_cache.has_key(addresspair):
             (stamp, uuid) = self.session_cache[addresspair]
             if time.time() - stamp < 60:
@@ -256,7 +286,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
                 raise MetadataConsistencyError
             return self.addresses[address][0]
         try:
-            return socket.gethostbyaddr(address)[0]
+            return socket.gethostbyaddr(address)[0].lower()
         except socket.herror:
             warning = "address resolution error for %s" % (address)
             self.logger.warning(warning)
@@ -264,6 +294,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
     
     def get_metadata(self, client):
         '''Return the metadata for a given client'''
+        client = client.lower()
         if self.aliases.has_key(client):
             client = self.aliases[client]
         if self.clients.has_key(client):
@@ -272,7 +303,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
             if self.default == None:
                 self.logger.error("Cannot set group for client %s; no default group set" % (client))
                 raise MetadataConsistencyError
-            self.set_profile(client, self.default)
+            self.set_profile(client, self.default, (None, None))
             [bundles, groups, categories] = self.groups[self.default]
         toolinfo = [self.toolsets[group] for group in groups if self.toolsets.has_key(group)]
         if len(toolinfo) > 1:
@@ -306,10 +337,12 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
         return ClientMetadata(client, newgroups, newbundles, toolset, newcategories,
                               probed, uuid, password)
         
-    def GetProbes(self, _):
+    def GetProbes(self, meta, force=False):
         '''Return a set of probes for execution on client'''
         ret = []
-        if self.probes:
+        ctime = time.time() - self.ptimes.get(meta.hostname, 0)
+        diff = ctime > self.pctime
+        if self.probes and (diff or force):
             bangline = re.compile('^#!(?P<interpreter>(/\w+)+)$')
             for name, entry in [(name, entry) for name, entry in \
                                 self.probes.entries.iteritems() if entry.data]:
@@ -322,7 +355,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
                     probe.set('interpreter', match.group('interpreter'))
                 else:
                     probe.set('interpreter', '/bin/sh')
-                    ret.append(probe)
+                ret.append(probe)
         return ret
 
     def ReceiveData(self, client, data):
@@ -334,6 +367,8 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
                               (data.get('name'), client.hostname))
             return
         dlines = data.text.split('\n')
+        self.logger.debug("%s:probe:%s:%s" % (client.hostname, 
+            data.get('name'), [line.strip() for line in dlines]))
         for line in dlines[:]:
             if line.split(':')[0] == 'group':
                 newgroup = line.split(':')[1].strip()
@@ -345,6 +380,9 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
             self.probedata[client.hostname].update({ data.get('name'):dtext })
         except KeyError:
             self.probedata[client.hostname] = { data.get('name'):dtext }
+        self.pdirty = True
+        self.ptimes[client.hostname] = time.time()
+        self.write_probedata()
 
     def AuthenticateConnection(self, user, password, address):
         '''This function checks user and password'''
@@ -353,7 +391,8 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
             try:
                 client = self.resolve_client(address)
             except MetadataConsistencyError:
-                self.logger.error("Client %s failed to authenticate due to metadata problem" % (address))
+                self.logger.error("Client %s failed to authenticate due to metadata problem" % (address[0]))
+                return False
         else:
             # user maps to client
             if user not in self.uuid:
