@@ -1,7 +1,8 @@
 '''This module implements a templating generator based on Cheetah'''
-__revision__ = '$Revision: 2694 $'
+__revision__ = '$Revision: 3906 $'
 
-import logging, lxml.etree, posixpath, re, Cheetah.Parser, Cheetah.Template
+import logging, lxml.etree, posixpath, traceback, re, sys
+import Cheetah.Template, Cheetah.Parser
 import Bcfg2.Server.Plugin
 
 logger = logging.getLogger('Bcfg2.Plugins.TCheetah')
@@ -26,6 +27,11 @@ class TemplateFile:
             except Cheetah.Parser.ParseError, perror:
                 logger.error("Cheetah parse error for file %s" % (self.name))
                 logger.error(perror.report())
+        elif event.filename == 'info.xml':
+            if not hasattr(self, 'infoxml'):
+                fpath = self.name + '/info.xml'
+                self.infoxml = Bcfg2.Server.Plugin.XMLSrc(fpath, True)
+            self.infoxml.HandleEvent(event)
         elif event.filename == 'info':
             for line in open(self.name + '/info').readlines():
                 match = info.match(line)
@@ -46,13 +52,25 @@ class TemplateFile:
     def BuildFile(self, entry, metadata):
         '''Build literal file information'''
         self.template.metadata = metadata
+        self.template.path = entry.get('realname', entry.get('name'))
         self.template.properties = self.properties.properties
         try:
             entry.text = str(self.template)
         except:
+            (a,b,c) = sys.exc_info()
+            msg = traceback.format_exception(a, b, c, limit=2)[-1][:-1]
+            logger.error(msg)
             logger.error("Failed to template %s" % entry.get('name'))
+            del a, b, c
             raise Bcfg2.Server.Plugin.PluginExecutionError
-        [entry.attrib.__setitem__(key, value) for (key, value) in self.metadata.iteritems()]
+        if hasattr(self, 'infoxml'):
+            mdata = {}
+            self.infoxml.pnode.Match(metadata, mdata)
+            [entry.attrib.__setitem__(key, value) \
+             for (key, value) in mdata['Info'][None].iteritems()]
+        else:
+            [entry.attrib.__setitem__(key, value) \
+             for (key, value) in self.metadata.iteritems()]
 
 class CheetahProperties(Bcfg2.Server.Plugin.SingleXMLFileBacked):
     '''Class for Cheetah properties'''
@@ -72,7 +90,7 @@ class FakeProperties:
 class TCheetah(Bcfg2.Server.Plugin.Plugin):
     '''The TCheetah generator implements a templating mechanism for configuration files'''
     __name__ = 'TCheetah'
-    __version__ = '$Id: TCheetah.py 2694 2007-01-18 16:30:43Z desai $'
+    __version__ = '$Id: TCheetah.py 3906 2007-11-02 23:30:39Z desai $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
 
     def __init__(self, core, datastore):
@@ -101,7 +119,7 @@ class TCheetah(Bcfg2.Server.Plugin.Plugin):
         if event.filename[0] == '/':
             return
         epath = "".join([self.data, self.handles[event.requestID], event.filename])
-        if event.filename in ['info', 'template']:
+        if event.filename in ['info', 'template', 'info.xml']:
             identifier = self.handles[event.requestID][:-1]
         else:
             identifier = self.handles[event.requestID] + event.filename
