@@ -27,7 +27,7 @@ def config_item_modified(request, eyedee =None, timestamp = 'now'):
                    "WHERE reports_client.current_interaction_id = reports_modified_interactions.interaction_id "+
                    "AND reports_modified_interactions.interaction_id = reports_interaction.id "+
                    "AND reports_modified_interactions.modified_id = %s", [eyedee])
-        associated_client_list = Client.objects.filter(id__in=[x[0] for x in cursor.fetchall()])
+        associated_client_list = Client.objects.active(timestamp).filter(id__in=[x[0] for x in cursor.fetchall()])
     else:
         interact_queryset = Interaction.objects.interaction_per_client(timestamp)
         interactionlist = []
@@ -37,7 +37,7 @@ def config_item_modified(request, eyedee =None, timestamp = 'now'):
                    "WHERE reports_modified_interactions.interaction_id IN %s "+
                    "AND reports_modified_interactions.interaction_id = reports_interaction.id "+
                    "AND reports_modified_interactions.modified_id = %s", [interactionlist, eyedee])
-            associated_client_list = Client.objects.filter(id__in=[x[0] for x in cursor.fetchall()])
+            associated_client_list = Client.objects.active(timestamp).filter(id__in=[x[0] for x in cursor.fetchall()])
         else:
             associated_client_list = []
 
@@ -62,7 +62,7 @@ def config_item_bad(request, eyedee = None, timestamp = 'now'):
                    "WHERE reports_client.current_interaction_id = reports_bad_interactions.interaction_id "+
                    "AND reports_bad_interactions.interaction_id = reports_interaction.id "+
                    "AND reports_bad_interactions.bad_id = %s", [eyedee])
-        associated_client_list = Client.objects.filter(id__in=[x[0] for x in cursor.fetchall()])
+        associated_client_list = Client.objects.active(timestamp).filter(id__in=[x[0] for x in cursor.fetchall()])
     else:
         interact_queryset = Interaction.objects.interaction_per_client(timestamp)
         interactionlist = []
@@ -72,7 +72,7 @@ def config_item_bad(request, eyedee = None, timestamp = 'now'):
                            "WHERE reports_bad_interactions.interaction_id IN %s "+
                            "AND reports_bad_interactions.interaction_id = reports_interaction.id "+
                            "AND reports_bad_interactions.bad_id = %s", [interactionlist, eyedee])
-            associated_client_list = Client.objects.filter(id__in=[x[0] for x in cursor.fetchall()])
+            associated_client_list = Client.objects.active(timestamp).filter(id__in=[x[0] for x in cursor.fetchall()])
         else:
             associated_client_list = None
 
@@ -94,8 +94,7 @@ def bad_item_index(request, timestamp = 'now'):
     if timestamp == 'now':
         bad_kinds = dict([(x,x.kind) for x in Bad.objects.filter(interactions__in=
                            [c.current_interaction
-                            for c in Client.objects.all()]).distinct()])
-                             #this will need expiration support
+                            for c in Client.objects.active(timestamp)]).distinct()])
         kinds = list(Set(bad_kinds.values()))
         item_list_dict = dict([(x,[]) for x in kinds])
         for obj in bad_kinds:
@@ -104,8 +103,7 @@ def bad_item_index(request, timestamp = 'now'):
     else: #this isn't done yet
         bad_kinds = dict([(x,x.kind) for x in Bad.objects.filter(interactions__in=
                            [c.current_interaction
-                            for c in Client.objects.all()]).distinct()])
-                             #this will need expiration support
+                            for c in Client.objects.active(timestamp)]).distinct()])
         kinds = list(Set(bad_kinds.values()))
         item_list_dict = dict([(x,[]) for x in kinds])
         for obj in bad_kinds:
@@ -128,7 +126,7 @@ def modified_item_index(request, timestamp = 'now'):
     if timestamp == 'now':
         mod_kinds = dict([(x,x.kind) for x in Modified.objects.filter(interactions__in=
                            [c.current_interaction
-                            for c in Client.objects.all()]).distinct()])
+                            for c in Client.objects.active(timestamp)]).distinct()])
                              #this will need expiration support
         kinds = list(Set(mod_kinds.values()))
         item_list_dict = dict([(x,[]) for x in kinds])
@@ -138,7 +136,7 @@ def modified_item_index(request, timestamp = 'now'):
     else: #this isn't done yet
         mod_kinds = dict([(x,x.kind) for x in Modified.objects.filter(interactions__in=
                            [c.current_interaction
-                            for c in Client.objects.all()]).distinct()])
+                            for c in Client.objects.active(timestamp)]).distinct()])
                              #this will need expiration support
         kinds = list(Set(mod_kinds.values()))
         item_list_dict = dict([(x,[]) for x in kinds])
@@ -156,12 +154,18 @@ def modified_item_index(request, timestamp = 'now'):
                                                             'timestamp_time' : timestamp[11:19]})
 
 
-def client_index(request):
-    client_list = Client.objects.all().order_by('name')
-    client_list_a = client_list[:len(client_list)/2]
-    client_list_b = client_list[len(client_list)/2:]
+def client_index(request, timestamp = 'now'):
+    timestamp = timestamp.replace("@"," ")
+    client_list = Client.objects.active(timestamp).order_by('name')
+    client_list_b = client_list[:len(client_list)/2]
+    client_list_a = client_list[len(client_list)/2:]
+    if timestamp == 'now':
+        timestamp = datetime.now().isoformat('@')
     return render_to_response('clients/index.html', {'client_list_a': client_list_a,
-                                                    'client_list_b': client_list_b})
+                                                     'client_list_b': client_list_b,
+                                                     'timestamp' : timestamp,
+                                                     'timestamp_date' : timestamp[:10],
+                                                     'timestamp_time' : timestamp[11:19]})
 
 def client_detail(request, hostname = None, pk = None):
     #SETUP error pages for when you specify a client or interaction that doesn't exist
@@ -172,6 +176,31 @@ def client_detail(request, hostname = None, pk = None):
         interaction = client.interactions.get(pk=pk)#can this be a get object or 404?
     return render_to_response('clients/detail.html', {'client': client, 'interaction': interaction})
 
+def client_manage(request, hostname = None):
+    #SETUP error pages for when you specify a client or interaction that doesn't exist
+    client = get_object_or_404(Client, name=hostname)
+    currenttime = datetime.now().isoformat('@')
+    if client.expiration != None:
+        message = "This client currently has an expiration date of %s. Reports after %s will not include data for this host. You may change this if you wish by selecting a new time, earlier or later."%(client.expiration,client.expiration)
+    else:
+        message = "This client is currently active and displayed. You may choose a date after which this client will no longer appear in reports."
+    if request.method == 'POST':
+        date = request.POST['date1']
+        time = request.POST['time']
+        try:
+            timestamp = datetime(*(strptime(date+"@"+time, "%Y-%m-%d@%H:%M:%S")[0:6]))
+        except ValueError:
+            timestamp = None
+        if timestamp==None:
+            message = "Invalid removal date, please try again using the format: yyyy-mm-dd hh:mm:ss."
+        else:
+            client.expiration = timestamp
+            client.save()
+            message = "Expiration for client set to %s."%client.expiration
+    return render_to_response('clients/manage.html', {'client': client, 'message': message,
+                                                      'timestamp_date' : currenttime[:10],
+                                                      'timestamp_time' : currenttime[11:19]})
+
 def display_sys_view(request, timestamp = 'now'):
     client_lists = prepare_client_lists(request, timestamp)
     return render_to_response('displays/sys_view.html', client_lists)
@@ -187,7 +216,7 @@ def display_timing(request, timestamp = 'now'):
     #+------+-------+----------------+-----------+---------+----------------+-------+
     #| name | parse | probe download | inventory | install | cfg dl & parse | total |
     #+------+-------+----------------+-----------+---------+----------------+-------+
-    client_list = Client.objects.all().order_by('name')
+    client_list = Client.objects.active(timestamp.replace("@"," ")).order_by('name')
     stats_list = []
 
     if not timestamp == 'now':
@@ -262,10 +291,10 @@ def prepare_client_lists(request, timestamp = 'now'):
     interact_queryset = Interaction.objects.interaction_per_client(timestamp)
     # or you can specify a time like this: '2007-01-01 00:00:00'
     [client_interaction_dict.__setitem__(x.client_id,x) for x in interact_queryset]
-    client_list = Client.objects.filter(id__in=client_interaction_dict.keys()).order_by('name')
+    client_list = Client.objects.active(timestamp).filter(id__in=client_interaction_dict.keys()).order_by('name')
 
-    [clean_client_list.append(x) for x in Client.objects.filter(id__in=[y.client_id for y in interact_queryset.filter(state='clean')])]
-    [bad_client_list.append(x) for x in Client.objects.filter(id__in=[y.client_id for y in interact_queryset.filter(state='dirty')])]
+    [clean_client_list.append(x) for x in Client.objects.active(timestamp).filter(id__in=[y.client_id for y in interact_queryset.filter(state='clean')])]
+    [bad_client_list.append(x) for x in Client.objects.active(timestamp).filter(id__in=[y.client_id for y in interact_queryset.filter(state='dirty')])]
 
     client_ping_dict = {}
     [client_ping_dict.__setitem__(x,'Y') for x in client_interaction_dict.keys()]#unless we know otherwise...
@@ -280,7 +309,7 @@ def prepare_client_lists(request, timestamp = 'now'):
 
     client_down_ids = [y for y in client_ping_dict.keys() if client_ping_dict[y]=='N']
     if not client_down_ids == []:
-        [down_client_list.append(x) for x in Client.objects.filter(id__in=client_down_ids)]
+        [down_client_list.append(x) for x in Client.objects.active(timestamp).filter(id__in=client_down_ids)]
 
     if (timestamp == 'now' or timestamp == None): 
         cursor.execute("select client_id, MAX(timestamp) as timestamp from reports_interaction GROUP BY client_id")
@@ -288,7 +317,7 @@ def prepare_client_lists(request, timestamp = 'now'):
         for x in results:
             if type(x[1]) == type(""):
                 x[1] = util.typecast_timestamp(x[1])
-        stale_all_client_list = Client.objects.filter(id__in=[x[0] for x in results if datetime.now() - x[1] > timedelta(days=1)])
+        stale_all_client_list = Client.objects.active(timestamp).filter(id__in=[x[0] for x in results if datetime.now() - x[1] > timedelta(days=1)])
     else:
         cursor.execute("select client_id, timestamp, MAX(timestamp) as timestamp from reports_interaction "+
                        "WHERE timestamp < %s GROUP BY client_id", [timestamp])
@@ -298,16 +327,16 @@ def prepare_client_lists(request, timestamp = 'now'):
         for x in results:
             if type(x[1]) == type(""):
                 x[1] = util.typecast_timestamp(x[1])
-        stale_all_client_list = Client.objects.filter(id__in=[x[0] for x in results if datetimestamp - x[1] > timedelta(days=1)])
+        stale_all_client_list = Client.objects.active(timestamp).filter(id__in=[x[0] for x in results if datetimestamp - x[1] > timedelta(days=1)])
         
     [stale_up_client_list.append(x) for x in stale_all_client_list if not client_ping_dict[x.id]=='N']
 
     
     cursor.execute("SELECT reports_client.id FROM reports_client, reports_interaction, reports_modified_interactions WHERE reports_client.id=reports_interaction.client_id AND reports_interaction.id = reports_modified_interactions.interaction_id GROUP BY reports_client.id")
-    modified_client_list = Client.objects.filter(id__in=[x[0] for x in cursor.fetchall()])
+    modified_client_list = Client.objects.active(timestamp).filter(id__in=[x[0] for x in cursor.fetchall()])
 
     cursor.execute("SELECT reports_client.id FROM reports_client, reports_interaction, reports_extra_interactions WHERE reports_client.id=reports_interaction.client_id AND reports_interaction.id = reports_extra_interactions.interaction_id GROUP BY reports_client.id")
-    extra_client_list = Client.objects.filter(id__in=[x[0] for x in cursor.fetchall()])
+    extra_client_list = Client.objects.active(timestamp).filter(id__in=[x[0] for x in cursor.fetchall()])
 
     if timestamp == 'now':
         timestamp = datetime.now().isoformat('@')
