@@ -1,14 +1,14 @@
 '''This file manages the statistics collected by the BCFG2 Server'''
-__revision__ = '$Revision: 2799 $'
+__revision__ = '$Revision: 3608 $'
 
 from lxml.etree import XML, SubElement, Element, XMLSyntaxError
-from time import asctime, localtime, time
+from time import asctime, localtime, time, strptime, mktime
 
 import logging, lxml.etree, os
 
 class Statistics(object):
     '''Manages the memory and file copy of statistics collected about client runs'''
-    __min_write_delay__ = 30
+    __min_write_delay__ = 0
 
     def __init__(self, filename):
         self.filename = filename
@@ -53,8 +53,9 @@ class Statistics(object):
         '''Updates the statistics of a current node with new data'''
 
         # Current policy: 
-        # - Keep latest clean run for clean nodes
-        # - Keep latest clean and dirty run for dirty nodes
+        # - Keep anything less than 24 hours old
+        #   - Keep latest clean run for clean nodes
+        #   - Keep latest clean and dirty run for dirty nodes
         newstat =  xml.find('Statistics')
 
         if newstat.get('state') == 'clean':
@@ -72,13 +73,12 @@ class Statistics(object):
             node = SubElement(self.element, 'Node', name=client)
         elif nummatch == 1 and not node_dirty:
             # Delete old instance
-            self.element.remove(nodes[0])
-            node = SubElement(self.element, 'Node', name=client)
+            node = nodes[0]
+            [node.remove(elem) for elem in node.findall('Statistics') if self.isOlderThan24h(elem.get('time'))]
         elif nummatch == 1 and node_dirty:
             # Delete old dirty statistics entry
             node = nodes[0]
-            for elem in [elem for elem in node.findall('Statistics') if elem.get('state') == 'dirty']:
-                node.remove(elem)
+            [node.remove(elem) for elem in node.findall('Statistics') if (elem.get('state') == 'dirty' and self.isOlderThan24h(elem.get('time')))]
         else:
             # Shouldn't be reached
             self.logger.error("Duplicate node entry for %s"%(client))
@@ -92,3 +92,13 @@ class Statistics(object):
         # Set dirty
         self.dirty = 1
         self.WriteBack()
+
+
+    def isOlderThan24h(self, testTime):
+        '''Helper function to determine if <time> string is older than 24 hours'''
+        now = time()
+        utime = mktime(strptime(testTime))
+        secondsPerDay = 60*60*24
+
+        return (now-utime) > secondsPerDay
+
